##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Basic portal calendar tool.
$Id: CalendarTool.py,v 1.25 2002/03/20 10:14:47 gitte Exp $
"""
__version__='$Revision: 1.25 $'

import os, zLOG, random, calendar
from Globals import InitializeClass, DTMLFile, package_home
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime

try:
    from Products.Localizer import Gettext
    use_localizer = 1
except:
    use_localizer = 0

_dtmldir = os.path.join(package_home(globals()), 'dtml')

if use_localizer:
    _ = Gettext.translation(globals())
    N_ = Gettext.dummy

    #Date Strings to be translated
    N_('Monday')
    N_('Tuesday')
    N_('Wedneysday')
    N_('Thursday')
    N_('Friday')
    N_('Saturday')
    N_('Sunday')

    #Short date strings to be translated
    N_('Mo')
    N_('Tu')
    N_('We')
    N_('Th')
    N_('Fr')
    N_('Sa')
    N_('Su')

    #Month Strings to be translated
    N_('January')
    N_('February')
    N_('March')
    N_('April')
    N_('May')
    N_('June')
    N_('July')
    N_('August')
    N_('September')
    N_('October')
    N_('November')
    N_('December')

class MediaManagerException( Exception ):
    pass

class CalendarTool( UniqueObject, SimpleItem ):

    """
    A tool for displaying months and dates
    """

    id = 'portal_calendar'
    meta_type = 'Calendar Tool'

    security = ClassSecurityInfo()
    
    queryPortalTypes = {}

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                     ,
                     ) + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'explainCalendarTool', _dtmldir)

    #
    #   'portal_mediamanager' interface methods
    #
    
    security.declarePublic('setDisplayMode')
    def setDisplayMode(self, mode):
        """
            Sets the display mode for the calendar.
            The options are 'All items' or 'Own items'
        """
        memtool = getToolByName(self, 'portal_membership')
        home = memtool.getHomeFolder()
        home.calendarDisplayMode = mode
        
    security.declarePublic('getUserMode')
    def getUserMode(self, object):
        """
            Returns the display mode for a specific object (usually this would be the desktop).
            The options are 'All items' or 'Own items'
        """
        memtool = getToolByName(self, 'portal_membership')
        creator = object.Creator()
        portal = getToolByName(self, 'portal_url').getPortalObject()
        path = portal.id + '/Members/' + creator
        home = self.restrictedTraverse(path)
        if hasattr(home, 'calendarDisplayMode'):
            return home.calendarDisplayMode
        else:
            return ''
    
    security.declarePublic('setCalendarTypes')
    def setCalendarTypes(self, types):
        """
            Set which portal types should be displayed in the calendar
        """
        memtool = getToolByName(self, 'portal_membership')
        home = memtool.getHomeFolder()
        home.calendarTypes = types
                
    security.declarePublic('getCalendarTypes')
    def getCalendarTypes(self):
        """
            Return the list of portal types that should be displayed in the calendar.
        """
        memtool = getToolByName(self, 'portal_membership')
        home = memtool.getHomeFolder()
        if hasattr(home, 'calendarTypes'):
            typelist = home.calendarTypes
        else:
            typelist = None
        return typelist
        
    security.declarePublic('queryCatalog')
    def queryCatalog(self, date, object):
        """
            Returns a list of 'brains' objects from the portal catalog based on the portal types and display mode selected by the user.
        """
        catalog = getToolByName(self, 'portal_catalog')
        memtool = getToolByName(self, 'portal_membership')
        creator = object.Creator()
        portal = getToolByName(self, 'portal_url').getPortalObject()
        path = portal.id + '/Members/' + creator
        home = self.restrictedTraverse(path)
        zLOG.LOG('Calendar Tool', zLOG.INFO, 'Found Home Folder: %s' % home.id)
        result = None
        if hasattr(home, 'calendarTypes'):
            types = home.calendarTypes
            for type in types:
                if type == 'Live Event' or type == 'Event':
                    if hasattr(home, 'calendarDisplayMode'):
                        if home.calendarDisplayMode == 'all_items':
                            res = catalog.searchResults(Type=type
                                                      , sort_on='start_date'
                                                      , sort_order='reverse'
                                                      , start_date=date.latestTime()
                                                      , start_date_usage='range:max'
                                                      , end_date=date.earliestTime()
                                                      , end_date_usage='range:min')
			    zLOG.LOG('Calendar Tool', zLOG.INFO, 'little res: %s\nLarge res:%s' % (str(res), str(result)))                        
                        else:
                            res = catalog.searchResults(Type=type
                                                      , Creator=creator
				                      , sort_on='start_date'
				                      , sort_order='reverse'
				                      , start_date=date.latestTime()
				                      , start_date_usage='range:max'
				                      , end_date=date.earliestTime()
                                                      , end_date_usage='range:min')
 			    zLOG.LOG('Calendar Tool', zLOG.INFO, 'little res: %s\nlarge res: %s' % (str(res), str(result)))
                    else:
		        res = catalog.searchResults(Type=type
		                                  , Creator=creator
		    	                          , sort_on='start_date'
		    	                          , sort_order='reverse'
		    	                          , start_date=date.latestTime()
		    	                          , start_date_usage='range:max'
		    	                          , end_date=date.earliestTime()
		                                  , end_date_usage='range:min')
			zLOG.LOG('Calendar Tool', zLOG.INFO, 'little res: %s\nLarge res: %s' % (str(res), str(result)))
                else:
                    if hasattr(home, 'calendarDisplayMode'):
                        if home.calendarDisplayMode == 'all_items':
                            res = catalog.searchResults(Type=type
                                                      , sort_on='Date'
		                                      , sort_order='reverse'
		                                      , modified=[date.earliestTime(), date.latestTime()]
		                                      , modified_usage='range:min:max')
			    zLOG.LOG('Calendar Tool', zLOG.INFO, 'little res: %s\nLarge result: %s' % (str(res), str(result)))
                        else:
                            res = catalog.searchResults(Type=type
			                              , sort_on='Date'
			    		              , sort_order='reverse'
			    		              , Creator=creator
			    		              , modified=[date.earliestTime(), date.latestTime()]
			    		              , modified_usage='range:min:max')
			    zLOG.LOG('Calendar Tool', zLOG.INFO, 'little res: %s\nlarge resu: %s' % (str(res), str(result)))
                    else:
                        res = catalog.searchResults(Type=type
                                                  , sort_on='Date'
					          , sort_order='reverse'
					          , Creator=creator
					          , modified=[date.earliestTime(), date.latestTime()]
					          , modified_usage='range:min:max')
			zLOG.LOG('Calendar Tool', zLOG.INFO, 'little res: %s\nLarge res: %s' % (str(res), str(result)))
                if not result:
                    result = res
                else:
                    if res:
                        #result.append(res)
                        result = result + res
        zLOG.LOG('Calendar Tool', zLOG.INFO, 'Query res: %s' % str(result))
        return result

    security.declarePublic('getPrevDate')
    def getPrevDate(self, date=None, mode='month'):
        """
            Return the previous date in the listing (a 'go back'function)
        """
        if date is None:
            date = DateTime()
        else:
            date = DateTime(date)

        year = date.year()
        month = date.month()
        day = date.day()
        
        if mode == 'day':
            rangem = month - 1
            monthrange = calendar.monthrange(year, rangem)
            lastday = monthrange[1]
            if month == 1 and day == 1:
                nyear = year - 1
                nmonth = 12
                nday = 31
            elif day == 1:
                nyear = year
                nmonth = month - 1
                nday = lastday
            else:
                nyear = year
                nmonth = month
                nday = day - 1

            ndate = DateTime(nyear, nmonth, nday)
            
        elif mode == 'month':
            if month == 1:
                nyear = year - 1
                nmonth = 12
            else:
                nyear = year
                nmonth = month - 1
            ndate = DateTime(nyear, nmonth, 1)
            
        return ndate

    security.declarePublic('getNextDate')
    def getNextDate(self, date=None, mode='month'):
        """
            Return the next date in the listing (a 'go forward' function)
        """
        if date is None:
            date = DateTime()
        else:
            date = DateTime(date)

        year = date.year()
        month = date.month()
        day = date.day()

        if mode == 'day':
            monthrange = calendar.monthrange(year, month)
            lastday = monthrange[1]
            if month == 12 and day == 31:
                nyear = year + 1
                nmonth = 1
                nday = 1
            elif day == monthrange[1]:
                nyear = year
                nmonth = month + 1
                nday = 1
            else:
                nyear = year
                nmonth = month
                nday = day + 1 
            ndate = DateTime(nyear, nmonth, nday)

        elif mode == 'month':
            if month == 12:
                nyear = year + 1
                nmonth = 1
            else:
                nyear = year
                nmonth = month + 1
            ndate = DateTime(nyear, nmonth, 1)
        
        return ndate

    security.declarePublic('getMonthName')
    def getHeadline(self, date=None, mode='month'):
        """
            Return the headline for the calendar.
        """
        if date is None:
            date = DateTime()
        else:
            date = DateTime(date)

        if mode == 'day':
            dname = date.Day()
            dname = _(dname)
            daydate = date.dd()
            mname = date.Month()
            if use_localizer:
                mname = _(mname)
            zLOG.LOG('Calendar', zLOG.INFO, 'translated: %s' % mname)
            mname = str(dname) + ' ' + str(daydate) + ' ' + str(mname)

        elif mode == 'month':
            mname = date.Month()
            if use_localizer:
                mname = _(mname)
            year = date.year()
            mname = str(mname) + ' ' + str(year)

        return mname
    
    security.declarePublic('getDateList')
    def getDateList(self, date=None, mode='month'):
        """
            The date argument will contain the date you want a list of dates for. If date is None, today will be used.
            The mode argument will contain the mode you want a list of dates for. Mode is default set to month.
            'day' are also acceptable mode.
        """
    	if date is None:
            date = DateTime()
        else:
            date = DateTime(date)

        catalog = getToolByName(self, 'portal_catalog')

        list = {}

        if mode == 'day':
            dayname = date.Day()
            if use_localizer:
                dayname = _(dayname)
            daynames = []
            list['daynames'] = daynames.append(dayname)
            montharr = []
            week = []
            day = {}
            daynr = date.day()
            date = date
            day['daynr'] = daynr
            day['date'] = date
            week.append(day)
            montharr.append(week)
            list['daylist'] = montharr

        elif mode == 'month':
            if use_localizer:
                daynames = []
                daynames.append(_('Mo'))
                daynames.append(_('Tu'))
                daynames.append(_('We'))
                daynames.append(_('Th'))
                daynames.append(_('Fr'))
                daynames.append(_('Sa'))
                daynames.append(_('Su'))
            else:
                daynames = ['Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su']
            list['daynames'] = daynames
            year = int(date.year())
            month = int(date.mm())
            monthlist = calendar.monthcalendar(year, month)
            montharr= []
            for weekentry in monthlist:
                week = []
                for dayentry in weekentry:
                    day = {}
                    if dayentry == 0:
                        day['daynr'] = ''
                        day['date'] = None
                        week.append(day)
                    else:
                        day['daynr'] = dayentry
                        day['date'] = DateTime(year, month, dayentry)
                        week.append(day)
                montharr.append(week)
            list['daylist'] = montharr
        return list

    security.declarePublic('getMonthRange')
    def getMonthRange(self, date=None):
        """
            Return the range for the month (firstdate, lastdate).
            Return as a tuple.
        """
        if date is None:
            date = DateTime()

        y, m, d = date.year(), date.month(), date.day()
        first = DateTime(y, m, 1)
        monthrange = calendar.monthrange(y, m)
        last = DateTime(y, m, monthrange[1])
        return first.earliestTime(), last.latestTime()
        


InitializeClass( CalendarTool )
