##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
"""

ADD_CONTENT_PERMISSION = 'Add portal content'

#Globals
import Globals
import zLOG
from os.path import join
import os
import shutil

#Zope
from DateTime.DateTime import DateTime
from AccessControl import ClassSecurityInfo, getSecurityManager
from Persistence import Persistent
from Acquisition import aq_parent, aq_inner, aq_base

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.CMFCorePermissions import ManageProperties
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.utils import getToolByName, tuplize
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalFolder import PortalFolder

#Online PIM
from utils import *
from Contact import Contact
from Profile import Profile
from ContactContainer import ContactContainer
from EventContainer import EventContainer

factory_type_information = {
                            'id': 'Desktop',
                            'meta_type': 'PIM Desktop',
                            'description': 'Use desktop to organize your contacts, mails etc.',
                            'product': 'OnlinePIM',
                            'icon': 'desktop_icon.png',
                            'factory': 'addDesktop',
                            'filter_content_types': 0,
                            'immediate_view': 'desktop_view',
                            'actions': 
                                    ({'id': 'view',
                                      'name': 'View',
                                      'action': 'desktop_view',
                                      'permissions': ('View',),
                                      'category': 'object'},
                                     {'id': 'edit_desktop',
                                      'name': 'Edit Desktop',
                                      'action': 'desktop_edit_form',
                                      'permissions': ('Manage properties',),
                                      'category': 'object'})
                          }

class Desktop( PortalFolder
             , PortalContent
             , DefaultDublinCoreImpl
          	 ):
    """
    A Desktop
    """

    meta_type='PIM Desktop'
    portal_type='Object'
    effective_date = expiration_date = None
    _isDiscussable = 1
    _ratingUsers = {}
    _Rating = -1
    Rating = -1

    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , description=''
                ):
        DefaultDublinCoreImpl.__init__(self, id, title)
        self.id = id
        self.description = description
        self.modules = []
        self._ratingUsers = {}
        self._Rating = -1
        self.Rating = -1

    security.declarePrivate('_initDesktop')
    def _initDesktop(self, REQUEST=None, user=None):
        owner = self.Creator()
        zLOG.LOG('Desktop', zLOG.INFO, 'Creator: %s' % owner)
        memtool = getToolByName(self, 'portal_membership')
        if memtool.isAnonymousUser():
            member = user
            zLOG.LOG('Desktop', zLOG.INFO, 'User is not logged in')
            ct = Contact('my_contact_information')
            self._setObject('my_contact_information', ct)
            contact = self._getOb('my_contact_information')
            if hasattr(contact, '_setPortalTypeName'):
                contact._setPortalTypeName('Contact')
            zLOG.LOG('Desktop', zLOG.INFO, 'Set portal type')
            contact.changeOwnership(member)
            contact.manage_setLocalRoles(user.getId(), ['Owner'])
            contact.reindexObject()
            pro = Profile('my_profile')
            self._setObject('my_profile', pro)
            profile = self._getOb('my_profile')
            if hasattr(profile, '_setPortalTypeName'):
                profile._setPortalTypeName('Profile')
            profile.changeOwnership(member)
            profile.manage_setLocalRoles(user.getId(), ['Owner'])
            profile.reindexObject()
            ctcon = ContactContainer('my_contacts')
            self._setObject('my_contacts', ctcon)
            ctcontainer = self._getOb('my_contacts')
            if hasattr(ctcontainer, '_setPortalTypeName'):
                ctcontainer._setPortalTypeName('ContactContainer')
            ctcontainer.changeOwnership(member)
            ctcontainer.manage_setLocalRoles(user.getId(), ['Owner'])
            ctcontainer.edit(title='My Contacts')
            ctcontainer.reindexObject()
            evcon = EventContainer('my_events')
            self._setObject('my_events', evcon)
            evcontainer = self._getOb('my_events')
            if hasattr(evcontainer, '_setPortalTypeName'):
                evcontainer._setPortalTypeName('EventContainer')
            evcontainer.changeOwnership(member)
            evcontainer.manage_setLocalRoles(user.getId(), ['Owner'])
            evcontainer.edit(title='My Events')
            evcontainer.reindexObject()
        else:
            member = memtool.getMemberById(owner)
            if member is None:
                member = memtool.getAuthenticatedMember()
            zLOG.LOG('Desktop', zLOG.INFO, 'Username: %s' % (member.getUserName()))
            contactid = 'my_contact_information'
            self.invokeFactory(id=contactid, type_name='Contact')
            contact = self._getOb(contactid)
            profileid = 'my_profile'
            self.invokeFactory(id=profileid, type_name='Profile')
            ctcontainerid = 'my_contacts'
            self.invokeFactory(id=ctcontainerid, type_name='ContactContainer')
            ctcontainer = self._getOb('my_contacts')
            ctcontainer.edit(title='My Contacts')
            ctcontainer.reindexObject()
            evcontainerid = 'my_events'
            self.invokeFactory(id=evcontainerid, type_name='EventContainer')
            evcontainer = self._getOb('my_events')
            evcontainer.edit(title='My Events')
            evcontainer.reindexObject()
        if REQUEST is not None:
            if REQUEST.has_key('firstname'):
                firstname = REQUEST['firstname']
            else:
                firstname = ''
            if REQUEST.has_key('lastname'):
                lastname = REQUEST['lastname']
            else:
                lastname = ''
            if REQUEST.has_key('job_title'):
                job_title = REQUEST['job_title']
            else:
                job_title = ''
            if REQUEST.has_key('company'):
                company = REQUEST['company']
            else:
                company = ''
            if REQUEST.has_key('address'):
                address = REQUEST['address']
            else:
                address = ''
            if REQUEST.has_key('phone'):
                phone = REQUEST['phone']
            else:
                phone = ''
            if REQUEST.has_key('homepage'):
                homepage = REQUEST['homepage']
            else:
                homepage = ''
            if REQUEST.has_key('email'):
                email = REQUEST['email']
            else:
                email = ''
            title = firstname + ' ' + lastname
            contact.edit(title=title
                    , firstname = firstname
                    , lastname = lastname
                    , job_title = job_title
                    , company = company
                    , address = address
                    , phone = phone
                    , pri_email = email
                    , homepage = homepage)
    
    def SearchableText(self):
        """
        text for indexing
        """
        return "%s %s" % (self.title, self.description)


    security.declareProtected(ManageProperties, 'edit')
    def edit(self, title = None, description = None):
        """
        Edit the title and description for your desktop
        """
        if title is not None:
            self.setTitle(title)

        if description is not None:
            self.setDescription(description)

        self.reindexObject()

    security.declareProtected(ManageProperties, 'addObject')
    def addObject(self, object_typename, objectid):
        """
        Add a new folder or object to your desktop
        This function is obsolete ?
        """
        self.invokeFactory(id=objectid, type_name=object_typename)

    security.declareProtected(ManageProperties, 'deleteObject')
    def deleteObject(self, object_id):
        """
        Delete a folder or an object from your desktop
        Will we ever need this function ?
        """
        self._delObject(object_id)
	
    security.declarePublic('listContactContainers')
    def listContactContainers(self):
        """
        Return a list of all Contact Container in your desktop
        """
        containers = self.objectValues('PIM Contact Container')
        return containers
        
    security.declarePublic('listEventContainers')
    def listEventContainers(self):
        """
        Return a list of all Event Containers in your desktop
        """
        containers = self.objectValues('PIM Event Container')
        return containers

    security.declarePublic('getRatingGrades')
    def getRatingGrades(self):
        """
        Return a list of rating grades
        This will be obsolete when we get a Rating tool
        """
        grades = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10']
        return grades

    security.declareProtected(CMFCorePermissions.View, 'postRating')
    def postRating(self, user=None, rating=None):
        """
        Post a rating for the desktop
        This will be obsolete when we get a Rating tool
        """
        if user == 'Guest': return

        alreadyRatedIt = self._ratingUsers.get(user, None)
        today = DateTime()

        if alreadyRatedIt:
            today_days = long(today)
            rating_days = long(alreadyRatedIt[1])
            diff = today_days - rating_days
            if diff < 30: return

        self._ratingUsers[user] = (int(rating), today)
        self._computeRating()
        self.reindexObject()
        self._p_changed = 1

    security.declareProtected(CMFCorePermissions.View, '_computeRating')
    def _computeRating(self):
        """
        Calculate the rating for the desktop
        This will be obsolete when we get a Rating tool
        """
        ratings = self._ratingUsers.items()
        rLen = len(ratings)

        total = 0
        oldest = None

        for r in ratings:
	    total = total + int(r[1][0])
	    r_time = r[1][1].millis()
	    if oldest == None or r_time < oldest[1]:
	       oldest = (r[0], r_time)

        self._Rating = total / rLen
        self.Rating = self._Rating

        if rLen > 100:
            del self._ratingUsers[oldest[0]]
            self._p_changed = 1

    security.declarePublic('getRating')
    def getRating(self):
        """
        Return the rating tuple for the desktop
        This will be obsolete when we get a Rating tool
        """
        rLen = len(self._ratingUsers)
        return (self._Rating, rLen)

    security.declarePublic('Rating')
    def Rating(self):
        """
        Return the rating tuple for the desktop
        This will be obsolete when we get e rating tool
        """
        return self._Rating

    security.declareProtected(CMFCorePermissions.ManagePortal, 'changeOwner')
    def changeOwner(self, memberid):
        """
        A function for changing owner.
        Used by the MembershipTool class
        """
        acl_users = self.__getPUS()
        user = acl_users.getUser(memberid).__of__(acl_users)
        self.changeOwnership(user)
        self.manage_setLocalRoles(memberid, ['Owner'])
        profile = self.objectValues('PIM Profile')[0]
        profile.changeOwnership(user)
        profile.manage_setLocalRoles(memberid, ['Owner'])
	
    security.declarePublic('getProfile')
    def getProfile(self):
        """
        Return the profile object
        """
        try:
            profile = self.objectValues('PIM Profile')[0]
        except:
            profile = None
        return profile
        
    security.declarePublic('getContact')
    def getContact(self):
        """
        Return the contact object
        """
        try:
            contact = self.objectValues('PIM Contact')[0]
        except:
            contact = None
        return contact
        
    security.declarePublic('getContactFolders')
    def getContactFolders(self):
        """
        Return a list of Contact Containers created by the creator of the Desktop object.
        """
        catalog = getToolByName(self, 'portal_catalog')
        creator = self.Creator()
        folders = catalog.searchResults(meta_type='PIM Contact Container'
                                      , Creator=creator)
        return folders

def addDesktop(self, id, title='', REQUEST=None):
    ob = Desktop(id,title)
    self._setObject(id, ob)
    desktop = self._getOb(id)
    desktop._initDesktop(REQUEST)

Globals.InitializeClass(Desktop)

from Products.CMFCore.register import registerPortalContent
registerPortalContent(Desktop,
                      constructors=(addDesktop,),
		      action='desktop_edit_form',
		      icon='desktop_icon.png')
